import { Pipeline } from "../pipeline.js";
import { ASTRO_PAGE_RESOLVED_MODULE_ID } from "./plugins/plugin-pages.js";
import { RESOLVED_SPLIT_MODULE_ID } from "./plugins/plugin-ssr.js";
import { ASTRO_PAGE_EXTENSION_POST_PATTERN } from "./plugins/util.js";
import { getOutputDirectory, isServerLikeOutput } from "../../prerender/utils.js";
import { createEnvironment } from "../render/index.js";
import { BEFORE_HYDRATION_SCRIPT_ID } from "../../vite-plugin-scripts/index.js";
import { createAssetLink } from "../render/ssr-element.js";
class BuildPipeline extends Pipeline {
  #internals;
  #staticBuildOptions;
  #manifest;
  #currentEndpointBody;
  constructor(staticBuildOptions, internals, manifest) {
    const ssr = isServerLikeOutput(staticBuildOptions.settings.config);
    super(
      createEnvironment({
        adapterName: manifest.adapterName,
        logging: staticBuildOptions.logging,
        mode: staticBuildOptions.mode,
        renderers: manifest.renderers,
        clientDirectives: manifest.clientDirectives,
        compressHTML: manifest.compressHTML,
        async resolve(specifier) {
          const hashedFilePath = manifest.entryModules[specifier];
          if (typeof hashedFilePath !== "string") {
            if (specifier === BEFORE_HYDRATION_SCRIPT_ID) {
              return "";
            }
            throw new Error(`Cannot find the built path for ${specifier}`);
          }
          return createAssetLink(hashedFilePath, manifest.base, manifest.assetsPrefix);
        },
        routeCache: staticBuildOptions.routeCache,
        site: manifest.site,
        ssr,
        streaming: true
      })
    );
    this.#internals = internals;
    this.#staticBuildOptions = staticBuildOptions;
    this.#manifest = manifest;
    this.setEndpointHandler(this.#handleEndpointResult);
  }
  getInternals() {
    return this.#internals;
  }
  getSettings() {
    return this.#staticBuildOptions.settings;
  }
  getStaticBuildOptions() {
    return this.#staticBuildOptions;
  }
  getConfig() {
    return this.#staticBuildOptions.settings.config;
  }
  getManifest() {
    return this.#manifest;
  }
  /**
   * The SSR build emits two important files:
   * - dist/server/manifest.mjs
   * - dist/renderers.mjs
   *
   * These two files, put together, will be used to generate the pages.
   *
   * ## Errors
   *
   * It will throw errors if the previous files can't be found in the file system.
   *
   * @param staticBuildOptions
   */
  static async retrieveManifest(staticBuildOptions, internals) {
    const config = staticBuildOptions.settings.config;
    const baseDirectory = getOutputDirectory(config);
    const manifestEntryUrl = new URL(
      `${internals.manifestFileName}?time=${Date.now()}`,
      baseDirectory
    );
    const { manifest } = await import(manifestEntryUrl.toString());
    if (!manifest) {
      throw new Error(
        "Astro couldn't find the emitted manifest. This is an internal error, please file an issue."
      );
    }
    const renderersEntryUrl = new URL(`renderers.mjs?time=${Date.now()}`, baseDirectory);
    const renderers = await import(renderersEntryUrl.toString());
    if (!renderers) {
      throw new Error(
        "Astro couldn't find the emitted renderers. This is an internal error, please file an issue."
      );
    }
    return {
      ...manifest,
      renderers: renderers.renderers
    };
  }
  /**
   * It collects the routes to generate during the build.
   *
   * It returns a map of page information and their relative entry point as a string.
   */
  retrieveRoutesToGenerate() {
    const pages = /* @__PURE__ */ new Map();
    for (const [entryPoint, filePath] of this.#internals.entrySpecifierToBundleMap) {
      if (entryPoint.includes(ASTRO_PAGE_RESOLVED_MODULE_ID) || entryPoint.includes(RESOLVED_SPLIT_MODULE_ID)) {
        const [, pageName] = entryPoint.split(":");
        const pageData = this.#internals.pagesByComponent.get(
          `${pageName.replace(ASTRO_PAGE_EXTENSION_POST_PATTERN, ".")}`
        );
        if (!pageData) {
          throw new Error(
            "Build failed. Astro couldn't find the emitted page from " + pageName + " pattern"
          );
        }
        pages.set(pageData, filePath);
      }
    }
    for (const [path, pageData] of this.#internals.pagesByComponent.entries()) {
      if (pageData.route.type === "redirect") {
        pages.set(pageData, path);
      }
    }
    return pages;
  }
  async #handleEndpointResult(request, response) {
    if (response.type === "response") {
      if (!response.response.body) {
        return new Response(null);
      }
      const ab = await response.response.arrayBuffer();
      const body = new Uint8Array(ab);
      this.#currentEndpointBody = {
        body,
        encoding: "utf-8"
      };
      return response.response;
    } else {
      if (response.encoding) {
        this.#currentEndpointBody = {
          body: response.body,
          encoding: response.encoding
        };
        const headers = new Headers();
        headers.set("X-Astro-Encoding", response.encoding);
        return new Response(response.body, {
          headers
        });
      } else {
        return new Response(response.body);
      }
    }
  }
  async computeBodyAndEncoding(routeType, response) {
    const encoding = response.headers.get("X-Astro-Encoding") ?? "utf-8";
    if (this.#currentEndpointBody) {
      const currentEndpointBody = this.#currentEndpointBody;
      this.#currentEndpointBody = void 0;
      return currentEndpointBody;
    } else {
      return { body: await response.text(), encoding };
    }
  }
}
export {
  BuildPipeline
};
