import { Pipeline } from "../core/pipeline.js";
import { createEnvironment, loadRenderer } from "../core/render/index.js";
import { createResolve } from "./resolve.js";
import { RouteCache } from "../core/render/route-cache.js";
import { isServerLikeOutput } from "../prerender/utils.js";
import { Logger } from "../core/logger/core.js";
import mime from "mime";
import { attachCookiesToResponse } from "../core/cookies/index.js";
class DevPipeline extends Pipeline {
  #settings;
  #loader;
  #devLogger;
  #currentMatchedRoute;
  constructor({
    manifest,
    logging,
    settings,
    loader
  }) {
    const env = DevPipeline.createDevelopmentEnvironment(manifest, settings, logging, loader);
    super(env);
    this.#devLogger = new Logger(logging);
    this.#settings = settings;
    this.#loader = loader;
    this.setEndpointHandler(this.#handleEndpointResult);
  }
  setCurrentMatchedRoute(route) {
    this.#currentMatchedRoute = route;
  }
  clearRouteCache() {
    this.env.routeCache.clearAll();
  }
  getSettings() {
    return this.#settings;
  }
  getConfig() {
    return this.#settings.config;
  }
  getModuleLoader() {
    return this.#loader;
  }
  get logger() {
    return this.#devLogger;
  }
  async loadRenderers() {
    const renderers = await Promise.all(
      this.#settings.renderers.map((r) => loadRenderer(r, this.#loader))
    );
    this.env.renderers = renderers.filter(Boolean);
  }
  static createDevelopmentEnvironment(manifest, settings, logging, loader) {
    const mode = "development";
    return createEnvironment({
      adapterName: manifest.adapterName,
      logging,
      mode,
      // This will be overridden in the dev server
      renderers: [],
      clientDirectives: manifest.clientDirectives,
      compressHTML: manifest.compressHTML,
      resolve: createResolve(loader, settings.config.root),
      routeCache: new RouteCache(logging, mode),
      site: manifest.site,
      ssr: isServerLikeOutput(settings.config),
      streaming: true
    });
  }
  async #handleEndpointResult(_, result) {
    if (result.type === "simple") {
      if (!this.#currentMatchedRoute) {
        throw new Error(
          "In development mode, you must set the current matched route before handling a endpoint."
        );
      }
      let contentType = "text/plain";
      const filepath = this.#currentMatchedRoute.pathname || this.#currentMatchedRoute.segments.map((segment) => segment.map((p) => p.content).join("")).join("/");
      const computedMimeType = mime.getType(filepath);
      if (computedMimeType) {
        contentType = computedMimeType;
      }
      const response = new Response(
        result.encoding !== "binary" ? Buffer.from(result.body, result.encoding) : result.body,
        {
          status: 200,
          headers: {
            "Content-Type": `${contentType};charset=utf-8`
          }
        }
      );
      attachCookiesToResponse(response, result.cookies);
      return response;
    }
    return result.response;
  }
}
export {
  DevPipeline as default
};
